"""
    All Global Constants
"""

EXECUTION_START_TIME = '__execution_start_time_ms'
EXECUTION_END_TIME = '__execution_end_time_ms'

SSL_VERIFY_KEY = '__lh_ssl_verify'
THREADS_TO_USE = '__lh_multithread_count'
PATH_TO_CA_CERT = 'CURL_CA_BUNDLE'
TIME_BETWEEN_ROWS = "time_between_consecutive_requests_milliseconds"
